#ifndef CRYPT_H
#define CRYPT_H

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

#define SERPENT_SKEYSIZE    (132*sizeof(uint32_t))
#define SERPENT_BLOCKSIZE  16

#define SHA384_SIZE         (384/8)
#define SHA512_SIZE         (512/8)
#define HMAC384_SIZE        SHA384_SIZE

#define CRC32_SIZE          (32/8)

void      serpent_init    (uint8_t       *skey,
                           uint8_t const *key,
                           size_t         key_size);
void      serpent_cbc_enc (uint8_t const *skey,
                           uint8_t       *dst,
                           uint8_t const *src,
                           size_t         size);
void      serpent_cbc_dec (uint8_t const *skey,
                           uint8_t       *dst,
                           uint8_t const *src,
                           size_t         size);

void      sha384          (uint8_t *out,uint8_t const *data,size_t size);
void      sha512          (uint8_t *out,uint8_t const *data,size_t size);

void      hmac_sha384     (uint8_t       *out,
                           uint8_t const *key,
                           size_t         key_size,
                           uint8_t const *data,
                           size_t         data_size);

uint32_t  crc32           (uint8_t const *data,size_t size,uint32_t crc);

int       rng_open        (char const *path);
bool      rng_close       (int fd);
bool      rng_read        (int fd,void *data,size_t size);

bool      entropy_missing (uint8_t const *data,size_t size);

#endif

