#include "crypt.h"
#include "internal.h"
#include <util/util.h>
#include <string.h>

#define IPAD_CONST            0x36
#define OPAD_CONST            0x5C

void hmac_sha384(uint8_t       *out,
                 uint8_t const *key,
                 size_t         key_size,
                 uint8_t const *data,
                 size_t         data_size){

  uint8_t   key_block [SHA384_BLOCK_SIZE];
  uint8_t   inner     [SHA384_SIZE];
  uint64_t  state     [SHA512_STATE_N];

  if(key_size <= SHA384_BLOCK_SIZE){
    memcpy(key_block,key,key_size);
  }else{
    sha384(key_block,key,key_size);
    key_size = SHA384_SIZE;
  }

  for(size_t x = 0;x < key_size;x++){
    key_block[x] ^= IPAD_CONST;
  }
  memset(key_block+key_size,IPAD_CONST,SHA384_BLOCK_SIZE-key_size);

  memcpy(state,h384_init,sizeof(state));
  sha512_block(state,key_block);
  sha512_compute(state,1,data,data_size);
  sha512_output(inner,state,SHA384_OUT_N);

  for(size_t x = 0;x < SHA384_BLOCK_SIZE;x++){
    key_block[x] ^= IPAD_CONST ^ OPAD_CONST;
  }

  memcpy(state,h384_init,sizeof(state));
  sha512_block(state,key_block);
  sha512_compute(state,1,inner,SHA384_SIZE);
  sha512_output(out,state,SHA384_OUT_N);
  BUF_ZERO(key_block);
}

