#include "crypt.h"
#include <util/util.h>
#include <log/log.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>

int rng_open(char const *path){
  int fd = open(path,O_RDONLY);
  if(fd == -1){
    log_errno("open");
    log_err("failed to open rng %s",path);
    return -1;
  }
  return fd;
}

bool rng_close(int fd){
  if(close(fd) != 0){
    log_errno("close");
    log_err("failed to close rng");
  }
  return 1;
}

static ssize_t _rng_read(int fd,void *data,size_t size){
  char   *p       = data;
  size_t  p_size  = size;

  memset_ffs(p,0x00,p_size);

  do{
    ssize_t n = read(fd,p,p_size);
    if(n < 0)               return -1;
    if(n > (ssize_t)p_size) return  0;
    p       += n;
    p_size  -= n;
  }while(p_size > 0);

  return size;
}

bool rng_read(int fd,void *data,size_t size){
  ssize_t r = _rng_read(fd,data,size);
  if(r != (ssize_t)size){
    if(r == -1) log_errno("read");
    log_err("failed to read %zu bytes from rng",size);
    return 0;
  }
  return 1;
}

bool entropy_missing(uint8_t const *data,size_t size){
  uint8_t first = data[0];
  for(size_t x = 1;x < size;x++){
    if(data[x] != first) return 0;
  }
  return 1;
}

