#include "daemon.h"
#include "internal.h"
#include "state/state.h"
#include <os.h>
#include <config.h> //MEMSET_RED
#include <net/ipv4.h>
#include <net/util.h>
#include <io/io.h>
#include <log/log.h>
#include <util/util.h>
#include <util/string.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#if defined(OS_LINUX)
  #include "linux.h"
  #include <sys/ioctl.h>
  #include <netinet/in.h>

  #define DEV_TUN "/dev/net/tun"
#else
  #include <net/if.h>

  #define DEV_TUN "/dev/tun"
#endif

int         dev_fd = -1;
static char ifname[IFNAMSIZ];

static bool dev_init_if(int fd){
#if defined(OS_LINUX)
  struct ifreq ifr;
  VAR_ZERO(ifr);
  ifr.ifr_flags = IFF_TUN | IFF_NO_PI;
  strzcpy(ifr.ifr_name,"gn%d",sizeof(ifr.ifr_name));
  if(ioctl(fd,TUNSETIFF,&ifr) == -1){
    log_errno("ioctl");
    log_err("failed to set tunnel device parameters");
    return 0;
  }
  strzcpy(ifname,ifr.ifr_name,sizeof(ifname));
#else
  char const *name = fdevname(fd);
  if(!name){
    log_err("failed to get device name");
    return 0;
  }
  strzcpy(ifname,name,sizeof(ifname));
#endif
  return 1;
}

static int dev_open(void){
  int fd = open(DEV_TUN,O_RDWR | O_NONBLOCK);
  if(fd == -1){
    log_errno("open");
    log_err("failed to open tunnel device %s",DEV_TUN);
    goto fail;
  }
  log_out("tunnel device %s opened",DEV_TUN);
    
  if(!dev_init_if(fd)) goto close;

  return fd;

close:
  close(fd);
fail:
  return -1;
}

static bool dev_close(int fd,bool silent){
  if(close(fd) != 0){
    log_errno("close");
    log_err("failed to close tunnel device");
    return 0;
  }

  if(!silent){
    log_out("tunnel device closed");
  }

  return 1;
}

bool dev_init(void){
  dev_fd = dev_open();
  return dev_fd != -1;
}

bool dev_exit(bool silent){
  if(!dev_close(dev_fd,silent)) return 0;
  dev_fd = -1;
  return 1;
}
 
void dev_recv(void){
  uint8_t buf[IPV4_MTU];
  size_t  size;

  switch(fd_read(dev_fd,
                 buf,
                 sizeof(buf),
                 &size)){

  case IO_OK:
    tx_ipv4(buf,size);
    break;

  case IO_FAIL:
#ifdef MEMSET_RED
    BUF_ZERO(buf);
#endif
    log_err("failed to read from tunnel device");
    break;

  case IO_BLOCK:
    break;
  }
}

char const *dev_ifname(void){
  return ifname;
}

