#ifndef DAEMON_INTERNAL_H
#define DAEMON_INTERNAL_H

#include "util.h"
#include "peer/peer.h"
#include "peer/key.h"
#include <crypt/crypt.h>
#include <net/util.h>

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#define IP_A(I) ipv4_fmt(ips_a,sizeof(ips_a),I)
#define IP_B(I) ipv4_fmt(ips_b,sizeof(ips_b),I)

enum{
  DIR_IN,
  DIR_OUT,
};

bool        ver_check       (uint8_t const *ipak,int dir);
bool        len_check       (uint8_t const *ipak,size_t ipak_size,int dir);
bool        sum_check       (uint8_t const *ipak,int dir);
bool        ttl_dec         (uint8_t *ipak,int dir);
void        ipv4_update_sum (uint8_t *ipak);

bool        key_gen         (uint8_t *key);

bool        id_valid        (char const *id,char const **err);

bool        ip_inuse        (uint32_t ip);
bool        ip_valid        (uint32_t ip,char const **err);
bool        ip_next         (uint32_t *ip);

bool        tun_write       (uint8_t const *ipak,size_t ipak_size);

void        pkey_used       (peer_t *p,pkey_t *pk);

void        set_sockaddr_ip (void *sa,uint32_t ip);

bool        addr_recv       (peer_t *p,uint8_t *l,size_t l_size);
bool        addr_flood      (peer_t *p);
void        addr_tick       (uint64_t ntime);

bool        flood_recv      (peer_t *p,uint8_t *f,size_t f_size);
bool        flood_brd       (peer_t        *p,
                             uint8_t        type,
                             uint8_t const *load,
                             size_t         load_size);
void        flood_tick      (uint64_t ntime);
void        flood_flush     (void);

bool        prod_recv       (peer_t  *p,
                             uint8_t *l,
                             size_t   l_size,
                             uint32_t rxseq);
bool        prod_send       (peer_t *p,uint32_t flags);
bool        prod_send_cold  (peer_t *p,uint32_t flags);
bool        prod_brd        (peer_t *p,uint32_t flags);
void        prod_all        (uint32_t flags);
void        prod_tick       (uint64_t ntime);

void        rekey_init      (rekey_t *rk);
bool        rekey_active    (rekey_t *rk);
bool        rekey_keyexists (rekey_t *rk);
bool        rekey_initiate  (peer_t  *p);
bool        rekey_recv      (peer_t  *p,
                             pkey_t  *pk,
                             uint8_t  type,
                             uint8_t *l,
                             size_t   l_size);
void        rekey_tick      (uint64_t ntime);

bool        rx_blk          (uint8_t       *b,
                             size_t         b_size,
                             addr_in_t      addr,
                             bool           matryoshka);
bool        tx_load         (uint8_t const *load,
                             size_t         load_size,
                             uint8_t        type,
                             peer_t        *p,
                             pkey_t        *pk,
                             addr_in_t      addr);
bool        tx_ipv4         (uint8_t *ipak,size_t ipak_size);

bool        if_up           (void);
bool        if_down         (void);
bool        if_enumbrd      (uint32_t *brd,size_t brd_max,size_t *brd_n);

bool        route_setup     (void);

bool        dev_init        (void);
bool        dev_exit        (bool silent);
void        dev_recv        (void);
char const *dev_ifname      (void);

bool        udp_init        (void);
bool        udp_exit        (void);
bool        udp_rebind      (void);
void        udp_recv        (void);

extern int  dev_fd;
extern int  udp_fd;

extern int  rng_slow_fd;
extern int  rng_fast_fd;

#endif

