#include "internal.h"
#include "state/state.h"
#include <net/ipv4.h>

bool ip_inuse(uint32_t ip){
  return state.ip == ip || peer_lookup_ip(ip);
}

bool ip_valid(uint32_t ip,char const **err){
  if(!ipv4_insubnet(ip,state.subnet)){
    if(err) *err = "not in subnet";
    return 0;
  }

  if(ip_inuse(ip)){
    if(err) *err = "already in use";
    return 0;
  }

  return 1;
}

bool ip_next(uint32_t *ip){
  uint32_t m = ipv4_maskip(state.subnet.mask);
  uint32_t p = state.subnet.ip;

  for(uint32_t x = 0;;x++){
    uint32_t i = x & ~m;
    if(x > 0 && i == 0) return 0;
    i |= p;
    if(ip_inuse(i)) continue;
    *ip = i;
    return 1;
  }
}

