#include "pak.h"
#include "internal.h"
#include <config.h> //MEMSET_RED, GREASYPAK_TRUNC
#include <util/serial.h>
#include <log/log.h>

#include <assert.h>

bool red_hdr(uint8_t *r,uint8_t type,pkey_t const *pk){
  if(!rng_read(rng_fast_fd,
               r+RED_NONCE_OFF,
               RED_NONCE_SIZE)){
    return 0;
  }

  set_u32le (r+RED_SEQNO_OFF, pkey_txnext(pk));
  set_u8le  (r+RED_PROTO_OFF, PAK_PROTO);
  set_u8le  (r+RED_TYPE_OFF,  type);
  set_u16le (r+RED_PAD_OFF,   0x0000);

  return 1;
}

void load_copy(uint8_t        *l,
               uint8_t const  *load,
               size_t          load_size){

  assert(RED_LOAD_VALIDSIZE(load_size));
  memcpy(l,load,load_size);
}

static void red_pad(uint8_t *r,
                    size_t   r_size,
                    size_t   load_size){

  assert(RED_VALIDSIZE(r_size));
  assert(RED_LOAD_VALIDSIZE(load_size));

  uint8_t *l = r+RED_LOAD_OFF;
  memset(l+load_size,0x00,RED_PADDING(r_size,load_size));
}

void blk_ciph(uint8_t *b,uint8_t const *r,size_t r_size,uint8_t const *key){
  assert(PAK_CIPH_VALID(r_size));
  serpent_cbc_enc(key,
                  b+BLK_RED_OFF,
                  r,
                  r_size);
}

void blk_deciph(uint8_t *r,uint8_t const *b,size_t r_size,uint8_t const *key){
  assert(PAK_CIPH_VALID(r_size));
  serpent_cbc_dec(key,
                  r,
                  b+BLK_RED_OFF,
                  r_size);
}

void blk_sign(uint8_t *b,size_t r_size,uint8_t const *key){
  hmac_sha384(b+BLK_SIG_OFF,
              key,
              PAK_SIGKEY_SIZE,
              b+BLK_RED_OFF,
              r_size);
}

bool blk_verify(uint8_t const *b,size_t r_size,uint8_t const *key){
  uint8_t sign[PAK_SIG_SIZE];
  hmac_sha384(sign,
              key,
              PAK_SIGKEY_SIZE,
              b+BLK_RED_OFF,
              r_size);
  return memcmp(sign,
                b+BLK_SIG_OFF,
                PAK_SIG_SIZE) == 0;
}

//b = blk bufer that contains load. evrything else overwritten
bool blk_prepare(uint8_t       *b,
                 size_t         load_size,
                 uint8_t        type,
                 pkey_t const  *pk,
                 size_t        *b_size_out){

  assert(RED_LOAD_VALIDSIZE(load_size));

  uint8_t  *r;
  size_t    r_size;
  size_t    b_size;

  r           = b+BLK_RED_OFF;
#ifdef GREASYPAK_TRUNC
  r_size      = RED_SIZE(load_size);
#else
  r_size      = RED_MAXSIZE;
#endif
  b_size      = BLK_SIZE(r_size);
  *b_size_out = b_size;

  red_pad(r,r_size,load_size);

  if(!red_hdr(r,type,pk)){
    log_err("failed to create red packet");
#ifdef MEMSET_RED
    memset_ffs(r,0x00,r_size);
#endif
    return 0;
  }

  blk_ciph(b,r,r_size,pk->ciph_serp);
  blk_sign(b,r_size,pk->key.sub.sign);
  return 1;
}

