#ifndef KEY_H
#define KEY_H

#include <config.h>
#include <crypt/crypt.h>

#include <stddef.h>
#include <stdint.h>
#include <assert.h>

#define KEY_SIGN_SIZE                  32
#define KEY_CIPH_SIZE                  32
#define KEY_SIZE                        (KEY_SIGN_SIZE+KEY_CIPH_SIZE)

#define RXBUF_SIZE                      (1<<RXBUF_POW) //must be power of 2

#define PKEY_SEQBIT            0x80000000
#define PKEY_HALF                       (PKEY_SEQBIT>>1)
#define PKEY_LIMIT                      (PKEY_SEQBIT-1)

typedef struct pkey{
  union{
    uint8_t         full      [KEY_SIZE];
    struct{
      uint8_t       sign      [KEY_SIGN_SIZE];
      uint8_t       ciph      [KEY_CIPH_SIZE];
    }sub;
  }key;
  uint8_t           ciph_serp [SERPENT_SKEYSIZE];
  uint8_t           rxbuf     [RXBUF_SIZE];
  uint32_t          rxseq;
  uint32_t          txseq;
  uint64_t          stamp;
}pkey_t;

void                pkey_init   (pkey_t *pk);
void                pkey_prep   (pkey_t *pk);
void                pkey_set    (pkey_t *pk,uint8_t const *key);

void                pkey_rzero  (pkey_t *pk);
bool                pkey_recv   (pkey_t *pk,uint32_t rxseq);
inline uint32_t     pkey_sent   (pkey_t *pk);
inline bool         pkey_self   (pkey_t *pk,uint32_t seq);
inline uint32_t     pkey_txnext (pkey_t const *pk);
inline bool         pkey_txhalf (pkey_t const *pk);
inline bool         pkey_txlim  (pkey_t const *pk);

bool                key_equal   (uint8_t const *a,uint8_t const *b);
void                key_copy    (uint8_t *dst,uint8_t const *src);
void                key_xor     (uint8_t       *dst,
                                 uint8_t const *a,
                                 uint8_t const *b);
uint32_t            key_crc     (uint8_t const *k);
void                key_crc_hex (char *buf,size_t buf_size,uint8_t const *k);

inline uint32_t pkey_sent(pkey_t *pk){
  assert(!pkey_txlim(pk));
  return ++pk->txseq;
}

inline bool pkey_self(pkey_t *pk,uint32_t seq){
  return (pk->rxseq ^ seq) & PKEY_SEQBIT;
}

inline uint32_t pkey_txnext(pkey_t const *pk){
  return pk->txseq+1;
}

inline bool pkey_txhalf(pkey_t const *pk){
  return (pk->txseq & ~PKEY_SEQBIT) >= PKEY_HALF;
}

inline bool pkey_txlim(pkey_t const *pk){
  return (pk->txseq & ~PKEY_SEQBIT) >= PKEY_LIMIT;
}

#endif

