#ifndef PEER_H
#define PEER_H

#include "key.h"
#include <def.h>
#include <config.h>
#include <daemon/util.h>
#include <net/def.h>

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

#define KEY_MAX             4

#define PEER_MUTE  0x00000001
#define PEER_BRD   0x00000002

typedef struct rekey{
  unsigned      state;
  uint64_t      ntime;
  uint64_t      nlast;
  size_t        pak_cnt;
  uint8_t       offer_self    [SHA512_SIZE];
  uint8_t       offer_peer    [SHA512_SIZE];
  uint8_t       slice_self    [KEY_SIZE];
  uint8_t       slice_peer    [KEY_SIZE];
  uint8_t       key_old       [KEY_SIZE];
  uint8_t       key_new       [KEY_SIZE];
}rekey_t;

typedef struct peer{
  //disk
  char          id[ID_BUF];
  uint32_t      ip;
  addr_t        locaddr;
  addr_t        extaddr;
  pkey_t        pkey[KEY_MAX];
  size_t        pkey_n;
  uint32_t      flags;

  uint64_t      grx_n;
  uint64_t      grx_b;
  uint64_t      gtx_n;
  uint64_t      gtx_b;
  uint64_t      nrx_n;
  uint64_t      nrx_b;
  uint64_t      ntx_n;
  uint64_t      ntx_b;
  uint64_t      rrx_n;
  uint64_t      rrx_b;
  uint64_t      rtx_n;
  uint64_t      rtx_b;
  uint64_t      drx_n;
  uint64_t      drx_b;
  uint64_t      dtx_n;
  uint64_t      dtx_b;

  bool          edit;

  //memory
  uint64_t      ntime;        //peer last seen you at
  uint64_t      prod_ntime;   //last periodic attempt or last txed
  uint64_t      prod_txntime; //last actually txed
  uint32_t      prod_rxseq;   //last recved prod rxseq
  uint32_t      prod_txseq;   //last transmitted prod
  uint64_t      addr_ntime;   //last periodic cast
  rekey_t       rekey;
}peer_t;

extern peer_t   peer[PEER_MAX];
extern size_t   peer_n;

void            peer_init       (peer_t *p);
bool            peer_save       (peer_t *p);
void            peer_edit       (peer_t *p);
bool            peer_flush      (peer_t *p);

peer_t         *peer_alloc      (void);
void            peer_add        (void);
bool            peer_del        (peer_t *p);
bool            peer_rename     (peer_t *p,char const *id,char const **err);

bool            peer_key_add    (peer_t *p,uint8_t const *key,uint32_t ip_ref);
void            peer_pkey_del   (peer_t *p,pkey_t *pk);
bool            peer_key_del    (peer_t *p,uint8_t const *key);
pkey_t         *peer_key_lookup (peer_t const *p,uint8_t const *key);
pkey_t         *peer_key_lookupc(peer_t const *p,uint32_t crc);
pkey_t         *peer_key_recent (peer_t const *p);

peer_t         *peer_lookup     (char const *id);
peer_t         *peer_lookup_ip  (uint32_t ip);
peer_t         *peer_lookup_key (uint8_t const *key);

void            peer_addr_set   (peer_t *p,addr_t addr,bool local);
void            peer_addr_auto  (peer_t *p,addr_t addr);

addr_t         *peer_effaddr    (peer_t const  *p,uint64_t time);
bool            peer_cancrypt   (peer_t const  *p,
                                 pkey_t       **pk,
                                 char const   **err);
bool            peer_cansend    (peer_t const  *p,
                                 pkey_t       **pk,
                                 addr_t       **addr,
                                 char const   **err);
bool            peer_canbrd     (peer_t const  *p,
                                 pkey_t       **pk,
                                 char const   **err);
bool            peer_cold       (peer_t *p,uint64_t time);
bool            peer_cold_loc   (peer_t *p,uint64_t time);

size_t          peer_idx        (peer_t const *p);

void            peer_grx        (peer_t *p,uint64_t b);
void            peer_gtx        (peer_t *p,uint64_t b);
void            peer_nrx        (peer_t *p,uint64_t b);
void            peer_ntx        (peer_t *p,uint64_t b);
void            peer_rrx        (peer_t *p,uint64_t b);
void            peer_rtx        (peer_t *p,uint64_t b);
void            peer_drx        (peer_t *p,uint64_t b);
void            peer_dtx        (peer_t *p,uint64_t b);

void            peer_init_all   (void);
bool            peer_load_all   (void);
bool            peer_flush_all  (void);
void            peer_prod_all   (uint32_t flags);

#endif

