#include "internal.h"
#include "pak.h"
#include "state/state.h"
#include "peer/peer.h"
#include <def.h>
#include <util/time.h>
#include <log/log.h>

#define BRD_MAX 32

bool prod_recv(peer_t *p,uint8_t *l,size_t l_size,uint32_t rxseq){
  uint32_t  flags;
  uint32_t  txseq;
  addr_t    addr;

  if(l_size < PROD_SIZE){
    log_err("prod packet from peer %s invalid size (%u)",
            p->id,l_size);
    return 0;
  }
  
  p->prod_rxseq = rxseq;

  flags         = get_u32le(l+PROD_FLAGS_OFF);
  txseq         = get_u32le(l+PROD_SEQNO_OFF);
  addr.ip       = get_u32le(l+PROD_IP_OFF);
  addr.port     = get_u16le(l+PROD_PORT_OFF);
  addr.stamp    = 0;

  if(flags & PROD_REQANS){
    prod_send(p,0);
  }

  //TODO: smal ring bufer of last txed
  //TODO: latency bit calc avg of x
  if(txseq == p->prod_txseq){
    p->ntime = p->prod_txntime; 
  }

  if(!addr_null(addr) && !state_addr_set(addr)){
    log_err("prod packet from peer %s contains local ip",p->id);
  }

  return 1;
}

static bool prod_load_send(peer_t    *p,
                           pkey_t    *pk,
                           addr_in_t  addr,
                           uint32_t   flags,
                           uint64_t   ntime){

  uint8_t   l[PROD_SIZE];
  uint32_t  txseq = pkey_txnext(pk);

  set_u32le(l+PROD_FLAGS_OFF, flags);
  set_u32le(l+PROD_SEQNO_OFF, p->prod_rxseq);
  set_u32le(l+PROD_IP_OFF,    p->extaddr.ip);
  set_u16le(l+PROD_PORT_OFF,  p->extaddr.port);

  if(!tx_load(l,
              sizeof(l),
              PAK_TYPE_PROD,
              p,
              pk,
              addr)){
    return 0;
  }

  p->prod_ntime   = ntime;
  p->prod_txntime = ntime;
  p->prod_txseq   = txseq;
  return 1;
}

bool prod_send(peer_t *p,uint32_t flags){
  pkey_t   *pk;
  addr_t   *addr;

  if(!peer_cansend(p,&pk,&addr,0)){
    return 1;
  }

  return prod_load_send(p,pk,addr_to_in(*addr),flags,mono_nsec());
}

bool prod_send_cold(peer_t *p,uint32_t flags){
  if(peer_cold(p,time_sec())){
    return prod_send(p,flags);
  }else{
    return 1;
  }
}

bool prod_brd(peer_t *p,uint32_t flags){
  pkey_t     *pk;
  uint32_t    brd[BRD_MAX];
  size_t      brd_n;
  addr_in_t   ai;
  uint64_t    ntime;
  bool        r = 1;

  if(!peer_canbrd(p,&pk,0)){
    return 1;
  }

  if(!if_enumbrd(brd,BRD_MAX,&brd_n)){
    log_err("failed to enumerate interface broadcast addresses");
    return 0;
  }

  ntime   = mono_nsec();
  ai.port = p->locaddr.port;
  for(size_t x = 0;x < brd_n;x++){
    ai.ip = brd[x];
    if(!prod_load_send(p,pk,ai,flags,ntime)) r = 0;
  }
  return r;
}

void prod_all(uint32_t flags){
  for(size_t x = 0;x < peer_n;x++){
    peer_t *p = peer+x;
    prod_send(p,flags);
  }
}

void prod_tick(uint64_t ntime){
  for(size_t x = 0;x < peer_n;x++){
    peer_t   *p = peer+x;
    bool      cold;
    bool      cold_loc;
    uint32_t  flags;
    uint64_t  stamp;

    if(ntime < p->prod_ntime+PROD_INTERVAL_MAX) continue;

    stamp         = time_sec();
    cold          = peer_cold(p,stamp);
    cold_loc      = peer_cold_loc(p,stamp);
    flags         = cold ? PROD_REQANS : 0;
    p->prod_ntime = ntime;

    prod_send(p,flags);
    if((p->flags & PEER_BRD) && cold_loc){
      prod_brd(p,PROD_REQANS);
    }
  }
}

