#include "internal.h"
#include <os.h>

#ifdef OS_LINUX
bool route_setup(void){
  return 1;
}
#else

#include "state/state.h"
#include <log/log.h>
#include <util/util.h>

#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <net/route.h>

//weird constant from /sbin/route program
#define M_BUF_SIZE 512

static char *nextaddr(char *b,char const *e,struct sockaddr_storage *so){
  size_t size = SA_SIZE(so);
  if(b+size > e){
    log_err("buffer overflow");
    return 0;
  }
  memcpy(b,so,size);
  return b+size;
}

bool route_setup(void){
  int                       fd;
  struct sockaddr_storage   so_dst;
  struct sockaddr_storage   so_gateway;
  struct sockaddr_storage   so_netmask;
  char                     *b;
  char const               *e;

  struct{
    struct rt_msghdr        rtm;
    char                    buf[M_BUF_SIZE];
  }m;

  VAR_ZERO(m);
  VAR_ZERO(so_dst);
  VAR_ZERO(so_gateway);
  VAR_ZERO(so_netmask);

  m.rtm.rtm_type      = RTM_ADD;
  m.rtm.rtm_flags     = RTF_STATIC | RTF_GATEWAY | RTF_UP;
  m.rtm.rtm_version   = RTM_VERSION;
  m.rtm.rtm_seq       = 1;
  m.rtm.rtm_addrs     = 1<<RTAX_DST;
  m.rtm.rtm_addrs    |= 1<<RTAX_GATEWAY;
  m.rtm.rtm_addrs    |= 1<<RTAX_NETMASK;

  set_sockaddr_ip((void *)&so_dst,      state.subnet.ip);
  set_sockaddr_ip((void *)&so_gateway,  state.ip);
  set_sockaddr_ip((void *)&so_netmask,  ipv4_maskip(state.subnet.mask));

  b = m.buf;
  e = m.buf+sizeof(m.buf);
  b = nextaddr(b,e,&so_dst);      if(!b) goto fail;
  b = nextaddr(b,e,&so_gateway);  if(!b) goto fail;
  b = nextaddr(b,e,&so_netmask);  if(!b) goto fail;
  m.rtm.rtm_msglen = b-(char *)&m;

  fd = socket(PF_ROUTE,SOCK_RAW,0);
  if(fd == -1){
    log_errno("socket");
    log_err("failed to create socket");
    goto fail;
  }
  
  if(write(fd,&m,m.rtm.rtm_msglen) == -1 && errno != EEXIST){
    log_errno("write");
    log_err("failed to write to routing socket");
    goto close;
  }

  close(fd);
  return 1;

close:
  close(fd);
fail:
  return 0;
}

#endif

