#include "cmd.h"
#include <stdarg.h>
#include <assert.h>

#undef CMD
#define CMD(S,L,D,U)\
  { .sym_name = S,\
    .func     = rpc_##L,\
    .desc     = D,\
    .usage    = U,\
  },

rpc_cmdinfo_t rpc_cmdinfo[] = {
  CMD("HELP",   help,   "get description of all or specific command",       "help [cmd]")
  CMD("STOP",   stop,   "shut down greasynet",                              "stop")
  CMD("IF",     if,     "get/set tunnel interface name",                    "if [name]")
  CMD("SUBNET", subnet, "get/set subnet of virtual network",                "subnet [subnet]")
  CMD("IP",     ip,     "get/set self/peer virtual ip",                     "ip id [ip]")
  CMD("PORT",   port,   "get/set udp port to bind for greasy traffic",      "port [port]")
  CMD("ID",     id,     "get/set peer id",                                  "id id [new_id]")
  CMD("LOCADDR",locaddr,"get/set peer local address",                       "locaddr id [addr]")
  CMD("EXTADDR",extaddr,"get/set self/peer external address",               "extaddr id [addr]")
  CMD("ADDR",   addr,   "set self/peer address (auto detect)",              "addr id addr")
  CMD("MUTE",   mute,   "get/set peer mute flag (disable all comms)",       "mute id [enable]")
  CMD("BRD",    brd,    "get/set peer broadcast flag (lan peer discovery)", "brd id [enable]")
  CMD("ADD",    add,    "add new peer",                                     "add id")
  CMD("DEL",    del,    "delete existing peer",                             "del id")
  CMD("ADDKEY", addkey, "add new key to peer",                              "addkey id key")
  CMD("DELKEY", delkey, "delete existing key from peer",                    "delkey id key")
  CMD("GENKEY", genkey, "generate new key using system rng and print key",  "genkey")
  CMD("REKEY",  rekey,  "generate and exchange new key with peer",          "rekey [id]")
  CMD("PROD",   prod,   "prod peer/all peers immediately",                  "prod [id]")
  CMD("NET",    net,    "display virtual network information",              "net [id]")
  CMD("STAT",   stat,   "display network statistics",                       "stat [id]")
  CMD("KEYS",   keys,   "display short key information crc32(sha512(key))", "keys [id]")
  CMD("FKEYS",  fkeys,  "display full key information",                     "fkeys [id]")
};

rpc_cmd_t           rpc_cmd[ARR_CNT(rpc_cmdinfo)];
size_t              rpc_cmd_n = ARR_CNT(rpc_cmd);

val_t               sym_self;

val_t usage(rpc_f func){
  for(size_t x = 0;x < rpc_cmd_n;x++){
    rpc_cmdinfo_t *i = rpc_cmdinfo+x;
    if(i->func != func) continue;
    return rpc_err("usage: %s",i->usage);
  }
  assert(0);
  return rpc_err("international error");
}

val_t arg_err(char const *arg,char const *err){
  return rpc_err("invalid %s arg: %s",arg,err);
}

val_t peer_noexist(char const *id){
  return rpc_err("peer %s doesn't exist",id);
}

bool get_peer(val_t target,peer_t **peer,val_t *verr){
  char const *id;
  char const *err;

  if(!_stringp(target)){
    if(verr) *verr = arg_err("id","must be string");
    return 0;
  }

  id = str_data(target);
  if(!id_valid(id,&err)){
    if(verr) *verr = arg_err("id",err);
    return 0;
  }

  *peer = peer_lookup(id);
  if(!*peer){
    if(verr) *verr = peer_noexist(id);
    return 0;
  }

  return 1;
}

bool get_target(val_t target,peer_t **peer,val_t *verr){
  if(_stringp(target)){
    return get_peer(target,peer,verr);
  }else if(_symbolp(target) && _eql(target,sym_self)){
    *peer = 0;
    return 1;
  }else{
    if(verr) *verr = arg_err("id","must be symbol SELF or peer id as string");
    return 0;
  }
}

val_t save_cfg(bool restart_prog,bool rebind_udp){
  state_edit();

  if(restart_prog){
    daemon_stop(EXIT_RESTART);
  }

  if(rebind_udp && !udp_rebind()){
    return rpc_err("failed to rebind udp");
  }

  if(state_save()){
    return val_ok;
  }else{
    return rpc_err("failed to save config file");
  }
}

val_t save_peer(peer_t *p){
  peer_edit(p);
  if(peer_save(p)){
    return val_ok;
  }else{
    return rpc_err("failed to save peer file");
  }
}

