#include "cmd.h"
#include <net/util.h>
#include <util/time.h>

static val_t net_self(void){
  val_t effaddr = addr_null(state.extaddr) ? NIL : addr_get(state.extaddr);
  return list(PP_ALIST,
              cons(sym("ID"),     sym_self),
              cons(sym("IP"),     ipv4_get(state.ip)),
              cons(sym("EFFADDR"),effaddr));
}

static val_t net_peer(peer_t *p,uint64_t time){
  addr_t *effp    = peer_effaddr(p,time);
  val_t   effaddr = effp ? addr_get(*effp) : NIL;
  return list(PP_ALIST,
              cons(sym("ID"),     str_get (p->id)),
              cons(sym("IP"),     ipv4_get(p->ip)),
              cons(sym("EFFADDR"),effaddr),
              cons(sym("MUTE"),   flag_get(PEER_MUTE, p->flags)),
              cons(sym("BRD"),    flag_get(PEER_BRD,  p->flags)));
}

static bool smaler(uint32_t ip,int64_t min,int64_t min_prev){
  return (int64_t)ip > min_prev &&
         (int64_t)ip < min;
}

static peer_t *min_next(int64_t *min_io){
  int64_t   min       = INT64_MAX;
  int64_t   min_prev  = *min_io;
  peer_t   *p_min     = 0;

  if(smaler(state.ip,min,min_prev)){
    p_min = 0;
    min   = state.ip;
  }

  for(size_t x = 0;x < peer_n;x++){
    peer_t *p = peer+x;
    if(smaler(p->ip,min,min_prev)){
      p_min = p;
      min   = p->ip;
    }
  }

  *min_io = min;
  return p_min;
}

val_t rpc_net(val_t arg,size_t n){
  uint64_t  time;
  val_t     lst;

  if(n > 1) return usage(rpc_net);

  time  = time_sec();
  lst   = list(PP_LIST);

  if(n == 0){
    int64_t min = -1;

    //1+peers == self+peers
    for(size_t x = 0;x < 1+peer_n;x++){
      peer_t *p = min_next(&min);
      if(p){
        push(net_peer(p,time),lst);
      }else{
        push(net_self(),lst);
      }
    }
  }else{
    peer_t *p;
    val_t   verr;

    if(!get_target(_nth(0,arg),&p,&verr)){
      return verr;
    }
    
    push(p ? net_peer(p,time) : net_self(),lst);
  }

  return list(sym("NET"),
              int_get(time),
              nreverse(lst));
}

