#include "cmd.h"
#include <daemon/internal.h>

static uint32_t new_ip(uint32_t ip,uint32_t np,uint32_t om,uint32_t nm){
  return (ip & ~(om | nm)) | np;
}

static bool new_ip_conflict(uint32_t ip,uint32_t *ni,size_t ni_n,size_t *idx){
  size_t n = 0;
  for(size_t x = 0;x < ni_n;x++){
    if(ip != ni[x]) continue;
    if(n > 0){
      *idx = x;
      return 1;
    }
    n++;
  }
  return 0;
}

val_t rpc_subnet(val_t arg,size_t n){
  subnet_t    sub;
  char const *err;
  uint32_t    oi[1+PEER_MAX]; //self+peer ips
  uint32_t    ni[1+PEER_MAX]; //self+peer ips
  size_t      ni_n;
  uint32_t    np;
  uint32_t    om;
  uint32_t    nm;
  char        sub_a[SUBNET_BUF];
  char        sub_b[SUBNET_BUF];

  if(n > 1) return usage(rpc_subnet);

  if(n == 0){
    return list(sym("SUBNET"),subnet_get(state.subnet));
  }

  if(!subnet_set(car(arg),&sub,&err)){
    return arg_err("subnet",err);
  }

  np    = sub.ip;
  om    = ipv4_maskip(state.subnet.mask);
  nm    = ipv4_maskip(sub.mask);
  ni_n  = 1+peer_n;

  oi[0] = state.ip;
  for(size_t x = 0;x < peer_n;x++){
    oi[1+x] = peer[x].ip;
  }

  for(size_t x = 0;x < ni_n;x++){
    ni[x] = new_ip(oi[x],np,om,nm);
  }

  for(size_t x = 0;x < ni_n;x++){
    size_t y;
    if(new_ip_conflict(ni[x],ni,ni_n,&y)){
      char ips_a[IPV4_BUF];
      char ips_b[IPV4_BUF];
      return rpc_err("ip %s would conflict with ip %s",
                     IP_A(oi[x]),IP_B(oi[y]));
    }
  }

  subnet_fmt(sub_a,sizeof(sub_a),state.subnet);
  subnet_fmt(sub_b,sizeof(sub_b),sub);
  log_out("subnet %s -> %s",sub_a,sub_b);

  state.subnet  = sub;
  state.ip      = ni[0];
  for(size_t x = 0;x < peer_n;x++){
    peer[x].ip = ni[1+x];
  }

  for(size_t x = 0;x < peer_n;x++){
    val_t r = save_peer(&peer[x]);
    if(!_eql(r,val_ok)) return r;
  }
  return save_cfg(1,0);
}

