#include "state.h"
#include "default.h"
#include <io/sx.h>
#include <daemon/type/type.h>
#include <daemon/internal.h>
#include <net/util.h>
#include <util/util.h>
#include <util/string.h>
#include <log/log.h>

#define SPEC_STRUCT (&state)
#define SPEC_FAIL   spec_fail

state_t state;

void state_init(void){
  strzcpy(state.ifa,DEF_IF,sizeof(state.ifa));
  state.subnet  = DEF_SUBNET;
  state.ip      = DEF_IP;
  state.port    = DEF_PORT;
  state.extaddr = DEF_EXTADDR;

  state.grx_n   = DEF_GRXN;
  state.grx_b   = DEF_GRXB;
  state.gtx_n   = DEF_GTXN;
  state.gtx_b   = DEF_GTXB;
  state.nrx_n   = DEF_NRXN;
  state.nrx_b   = DEF_NRXB;
  state.ntx_n   = DEF_NTXN;
  state.ntx_b   = DEF_NTXB;
  state.rrx_n   = DEF_RRXN;
  state.rrx_b   = DEF_RRXB;
  state.rtx_n   = DEF_RTXN;
  state.rtx_b   = DEF_RTXB;
  state.drx_n   = DEF_DRXN;
  state.drx_b   = DEF_DRXB;
  state.dtx_n   = DEF_DTXN;
  state.dtx_b   = DEF_DTXB;

  state.edit    = 0;
}

bool state_load(void){
  val_t         sx;
  lisp_frame_t  frame;
  char const   *err;
  char const   *err_opt;
  uint32_t      ip_tmp;

  state_init();

  lisp_store(&frame);
  if(!sx_load(CFG_PATH,&sx)) goto pop;

  lisp_try{
#include <daemon/type/spec_set.h>
#include "spec.h"
  }lisp_catch{
    log_lisperr();
    goto pop;
  }

  lisp_load(&frame);

  //TODO: this is kinda retarded
  ip_tmp    = state.ip;
  state.ip  = 0;
  if(!ip_valid(ip_tmp,&err)){
    char ip_buf[IPV4_BUF];
    ipv4_fmt(ip_buf,sizeof(ip_buf),ip_tmp);
    log_err("self ip %s %s",ip_buf,err);
    goto fail;
  }
  state.ip = ip_tmp;

  state.edit = 0;
  return state_save();

spec_fail:
  log_err("option %s: %s",err_opt,err);
pop:
  lisp_load(&frame);
fail:
  log_err("failed to load config file");
  return 0;
}

bool state_save(void){
  val_t         sx;
  lisp_frame_t  frame;

  lisp_store(&frame);
  lisp_try{
    sx = list(PP_ALIST);
#include <daemon/type/spec_get.h>
#include "spec.h"
    sx = nreverse(sx);
  }lisp_catch{
    log_lisperr();
    goto pop;
  }

  if(!sx_save(CFG_PATH,sx)) goto pop;
  lisp_load(&frame);
  state.edit = 0;
  return 1;

pop:
  lisp_load(&frame);
  log_err("failed to save config file");
  return 0;
}

void state_edit(void){
  state.edit = 1;
}

bool state_flush(void){
  if(!state.edit) return 1;
  return state_save();
}

bool state_addr_set(addr_t addr){
  char ips_a[IPV4_BUF];
  char ips_b[IPV4_BUF];

  if(addr_equal(addr,state.extaddr))  return 1;
  if(ipv4_local(addr.ip))             return 0;

  if(addr.ip != state.extaddr.ip){
    log_out("self external address %s:%u -> %s:%u",
            IP_A(state.extaddr.ip),
            state.extaddr.port,
            IP_B(addr.ip),
            addr.port);
  }

  state.extaddr = addr;
  state_edit();
  return 1;
}

//hak so wraps to 0 after 600 years imsted of error when store to disk
static int64_t wrap_to_int64(uint64_t i){
  return i & INT64_MAX;
}

#define STATS_IMPL(N)\
  void state_##N(uint64_t b){\
    state.N##_n = wrap_to_int64(state.N##_n+1);\
    state.N##_b = wrap_to_int64(state.N##_b+b);\
    state_edit();\
  }

STATS_IMPL(grx)
STATS_IMPL(gtx)
STATS_IMPL(nrx)
STATS_IMPL(ntx)
STATS_IMPL(rrx)
STATS_IMPL(rtx)
STATS_IMPL(drx)
STATS_IMPL(dtx)

