#ifndef STATE_H
#define STATE_H

#include <def.h>
#include <config.h>
#include <daemon/util.h>
#include <net/def.h>

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

typedef struct state{
  //disk
  char          ifa[IF_BUF];
  subnet_t      subnet;
  uint32_t      ip;
  uint16_t      port;
  addr_t        extaddr;

  uint64_t      grx_n; //g*  = gross udp payloads
  uint64_t      grx_b;
  uint64_t      gtx_n; //*n  = number of packets
  uint64_t      gtx_b; //*b  = total bytes
  uint64_t      nrx_n; //n*  = net encapsulated ipv4 traffic
  uint64_t      nrx_b;
  uint64_t      ntx_n;
  uint64_t      ntx_b;
  uint64_t      rrx_n; //rr* = rejected invalid udp, no key etc
  uint64_t      rrx_b;
  uint64_t      rtx_n; //rt* = rejected invalid ipv4, wrong dst ip etc
  uint64_t      rtx_b;
  uint64_t      drx_n; //dr* = discarded valid ipv4, rng read fail, overload etc
  uint64_t      drx_b;
  uint64_t      dtx_n; //dt* = discarded valid udp, system overload etc
  uint64_t      dtx_b;

  bool          edit;
}state_t;

extern state_t  state;

void            state_init      (void);
bool            state_load      (void);
bool            state_save      (void);

void            state_edit      (void);
bool            state_flush     (void);

bool            state_addr_set  (addr_t addr);

void            state_grx       (uint64_t b);
void            state_gtx       (uint64_t b);
void            state_nrx       (uint64_t b);
void            state_ntx       (uint64_t b);
void            state_rrx       (uint64_t b);
void            state_rtx       (uint64_t b);
void            state_drx       (uint64_t b);
void            state_dtx       (uint64_t b);

#endif

