#include "daemon.h"
#include "internal.h"
#include "pak.h"
#include "state/state.h"
#include <io/io.h>
#include <net/udp.h>
#include <log/log.h>

#include <stdlib.h>

int udp_fd = -1;

bool udp_init(void){
  udp_fd = udp_bind(state.port,
                    NET_NONBLOCK |
                    NET_REUSEADDR |
                    NET_BROADCAST);
  if(udp_fd == -1){
    log_err("failed to bind udp port %u",state.port);
    return 0;
  }
  log_out("bound udp port %u",state.port);
  return 1;
}

bool udp_exit(void){
  bool r = 1;
  if(udp_fd == -1) return 1;
  if(close(udp_fd) != 0){
    log_errno("close");
    log_err("failed to close udp socket");
    r = 0;
  }
  log_out("closed udp socket");
  udp_fd = -1;
  return r;
}

bool udp_rebind(void){
  if(!udp_exit()) goto fail;
  if(!udp_init()) goto fail;
  return 1;

fail:
  log_err("failed to rebind udp, shutting down");
  daemon_stop(EXIT_FAILURE);
  return 0;
}

void udp_recv(void){
  uint8_t   buf[BLK_MAXSIZE+1]; //+1 to detect oversize
  size_t    size;
  addr_in_t addr;

  switch(udp_recvfrom(udp_fd,
                      &addr,
                      buf,
                      sizeof(buf),
                      &size)){

  case IO_OK:
    rx_blk(buf,size,addr,0);
    break;

  case IO_FAIL:
    log_err("failed to receive from udp socket");
    break;

  case IO_BLOCK:
    break;
  }
}

