#include "internal.h"
#include <log/log.h>

FILE *fp_open(char const *path,char const *mode){
  FILE *f = fopen(path,mode);
  if(!f){
    log_errno("fopen");
    log_err("failed to open file %s",path);
  }
  return f;
}

bool fp_close(FILE *f){
  int r = fclose(f);
  if(r != 0){
    log_errno("fclose");
    log_err("failed to close file handle");
    return 0;
  }
  return 1;
}

bool fp_seek(FILE *f,size_t off,int origin){
  if(fseek(f,off,origin) != 0){
    log_errno("fseek");
    log_err("failed to seek in file");
    return 0;
  }
  return 1;
}

bool fp_tell(FILE *f,size_t *pos){
  intptr_t ipos = ftell(f);
  if(ipos < 0){
    log_errno("ftell");
    log_err("failed to get file stream position");
    return 0;
  }
  *pos = ipos;
  return 1;
}

bool fp_size(FILE *f,size_t *size){
  if(!fp_seek(f,0,SEEK_END))  return 0;
  if(!fp_tell(f,size))        return 0;
  if(!fp_seek(f,0,SEEK_SET))  return 0;
  return 1;
}

bool fp_read(FILE *f,void *data,size_t size){
  size_t r = fread(data,1,size,f);
  if(r != size){
    if(ferror(f)){
      log_errno("fread");
      log_err("failed to read from file");
      return 0;
    }else{
      log_err("unexpected eof reached");
      return 0;
    }
  }
  return 1;
}

bool fp_write(FILE *f,void const *data,size_t size){
  if(fwrite(data,1,size,f) != size){
    log_errno("fwrite");
    log_err("failed to write to file");
    return 0;
  }
  return 1;
}

