#include "io.h"
#include <net/util.h>
#include <log/log.h>

#include <errno.h>
#include <unistd.h>

extern bool errno_block (void);
extern bool errno_down  (void);

int fd_read(int      fd,
            uint8_t *buf,
            size_t   buf_size,
            size_t  *size_out){

  ssize_t n = read(fd,buf,buf_size);
  if(n == -1){
    if(errno_block()) return IO_BLOCK;
    log_errno("read");
    log_err("failed to read from file descriptor");
    return IO_FAIL;
  }

  if(size_out){
    *size_out = n;
  }

  return IO_OK;
}

int fd_write(int            fd,
             uint8_t const *buf,
             size_t         buf_size,
             size_t        *size_out){

  ssize_t n = write(fd,buf,buf_size);
  if(n == -1){
    if(errno_block()) return IO_BLOCK;
    log_errno("write");
    log_err("failed to write to file descriptor");
    return IO_FAIL;
  }

  if(size_out){
    *size_out = n;
  }

  return IO_OK;
}

