#define _POSIX_C_SOURCE 200809L
#define _DEFAULT_SOURCE 1

#include "def.h"
#include "config.h"
#include <daemon/daemon.h>
#include <client/client.h>
#include <lisp/lisp.h>
#include <util/string.h>
#include <log/log.h>

#include <stdlib.h>
#include <limits.h>
#include <sys/stat.h>

#define SET_PATH(B,S)       set_path(B,sizeof(B),S)
#define ABS_PATH(B,S)       abs_path(B,sizeof(B),S)

#define LISP_CONS        4096
#define LISP_SYM         1024
#define LISP_STR        65536

LISP_STATIC_CTX(ctx,LISP_CONS,LISP_SYM,LISP_STR);

char          data_path[PATH_BUF];

extern uid_t  uid;
extern gid_t  gid;

static int usage(int code){
  fputs(
    "usage: " PROG_NAME " [options]\n"
    "\n"
    "options:\n"
    "  -d path      data directory (default = '" DATA_PATH "')\n"
    "  -u uid       run as user\n"
    "  -g gid       run as group\n"
    "  -r           start in rpc client mode\n"
    "  -v           display version\n"
    "  -h           display usage\n"
    "\n"
    "unless the -r option is specified,\n"
    "start the greasynet daemon (root required).\n"
    "\n"
    ,stderr
  );
  return code;
}

static bool set_path(char *buf,size_t buf_size,char const *path){
  if(!strzcpy_fail(buf,path,buf_size)){
    log_err("%s path too long",path);
    return 0;
  }
  return 1;
}

static bool abs_path(char *buf,size_t buf_size,char const *path){
  char path_buf[PATH_MAX+1];
  if(!realpath(path,path_buf)){
    log_errno("realpath");
    log_err("failed to get absolute %s path",path);
    return 0;
  }
  return set_path(buf,buf_size,path_buf);
}

static void must_int(char const *str,char const *name){
  for(char c;c = *str++;){
    if(c >= '0' && c <= '9') continue;
    log_err("%s must be integer",name);
    exit(EXIT_FAILURE);
  }
}

int main(int argc,char **argv){
  int   opt;
  bool  mode_cli = 0;

  umask(~DATA_MODE);
  log_add(stderr,LOG_FLAGS_STDERR);

  if(!SET_PATH(data_path,DATA_PATH)) return EXIT_FAILURE;

  while((opt = getopt(argc,argv,"d:u:g:rvh")) != -1){
    switch(opt){
    case 'd':
      if(!ABS_PATH(data_path,optarg)) return EXIT_FAILURE;
      break;

    case 'u':
      must_int(optarg,"-u option");
      uid = strtoll(optarg,0,10);
      break;

    case 'g':
      must_int(optarg,"-g option");
      gid = strtoll(optarg,0,10);
      break;

    case 'r':
      mode_cli = 1;
      break;

    case 'v':
      log_out(PROG_FULLNAME);
      return EXIT_SUCCESS;

    case 'h':
      return usage(EXIT_SUCCESS);

    case '?':
    default:
      return usage(EXIT_FAILURE);
    }
  }

  argc -= optind;
  argv += optind;

  lisp_ctx(&ctx);

  return mode_cli ? client_run(argc,argv) :
                    daemon_run(argc,argv);
}

