#include "ipv4.h"
#include <stdlib.h>

extern uint8_t  ipv4_ver_get  (uint8_t const *pak);
extern uint16_t ipv4_ihl_get  (uint8_t const *pak);
extern uint16_t ipv4_len_get  (uint8_t const *pak);
extern void     ipv4_len_set  (uint8_t       *pak,uint16_t len);
extern uint8_t  ipv4_fla_get  (uint8_t const *pak);
extern void     ipv4_fla_set  (uint8_t       *pak,uint8_t  fla);
extern uint16_t ipv4_fra_get  (uint8_t const *pak);
extern void     ipv4_fra_set  (uint8_t       *pak,uint16_t fra);
extern uint8_t  ipv4_ttl_get  (uint8_t const *pak);
extern void     ipv4_ttl_set  (uint8_t       *pak,uint8_t  sum);
extern uint16_t ipv4_sum_get  (uint8_t const *pak);
extern void     ipv4_sum_set  (uint8_t       *pak,uint16_t sum);
extern uint32_t ipv4_src_get  (uint8_t const *pak);
extern uint32_t ipv4_dst_get  (uint8_t const *pak);

uint16_t ipv4_sum(uint8_t const *pak){
  uint32_t  sum = 0x0000;
  size_t    ihl = ipv4_ihl_get(pak);

  for(size_t x = 0;x < ihl;x += 2){
    sum += (uint32_t)get_u16be(pak+x);
  }
  sum -= (uint32_t)ipv4_sum_get(pak);

  if(sum>>16){
    sum = (sum & 0xFFFF)+(sum>>16);
  }

  return (uint16_t)~sum;
}

int ipv4_frag(uint8_t       *dst,
              uint8_t const *src,
              size_t         dst_size,
              size_t         src_size,
              size_t        *_frag_off,
              size_t        *dst_size_out){

  size_t          frag_off;
  bool            more;
  uint16_t        ihl;
  uint16_t        len;
  uint8_t         fla;
  uint16_t        fra;
  uint16_t        sum;
  uint8_t        *dstd;
  size_t          dstd_size;
  uint8_t const  *srcd;
  size_t          srcd_size;
  uint8_t const  *frag;
  size_t          frag_size;

  frag_off  = *_frag_off;
  more      = 0;

  ihl       = ipv4_ihl_get(src);
  fla       = ipv4_fla_get(src);
  fra       = ipv4_fra_get(src);

  dstd      = dst       + ihl;
  dstd_size = dst_size  - ihl;
  srcd      = src       + ihl;
  srcd_size = src_size  - ihl;
  frag      = srcd      + frag_off;
  frag_size = srcd_size - frag_off;

  assert(dst_size  >= ihl);
  assert(dstd_size >= IPV4_FRA_UNIT);

  if(frag_size > dstd_size){
    more      = 1;
    frag_size = dstd_size/IPV4_FRA_UNIT*IPV4_FRA_UNIT;
  }

  memcpy(dst,   src,  ihl);
  memcpy(dstd,  frag, frag_size);

  len   = ihl+frag_size;
  fra  += frag_off;
  ipv4_len_set(dst,len);
  ipv4_fra_set(dst,fra);

  if(more){
    fla  |= IPV4_FLA_MF;
    ipv4_fla_set(dst,fla);
  }

  sum = ipv4_sum(dst);
  ipv4_sum_set(dst,sum);

  *_frag_off     += frag_size;
  *dst_size_out   = len;
  return more ? IPV4_FRAG_MORE :
                IPV4_FRAG_LAST;
}

