#ifndef NET_UTIL_H
#define NET_UTIL_H

#include "def.h"

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#define ADDR_BUF              sizeof("255.255.255.255:65535")
#define UNIX_PATH_BUF       108

#define ADDR_IN_CONST(I,P)    ((addr_in_t){.ip    = (I),\
                                           .port  = (P)})

#define IPV4_BUF              sizeof("255.255.255.255")
#define IPV4_CONST(A,B,C,D)   ((uint32_t)(A) << 24 | \
                               (uint32_t)(B) << 16 | \
                               (uint32_t)(C) <<  8 | \
                               (uint32_t)(D) <<  0)

#define IPV4_BROADCAST(I,M)   ((I) | 0xFFFFFFFFU>>(M))

#define SUBNET_BUF            sizeof("255.255.255.255/32")
#define SUBNET_CONST(I,M)     ((subnet_t){.ip   = (I),\
                                          .mask = (M)})

size_t      addr_in       (void    *sa,addr_in_t  addr);
void        addr_in_out   (void    *sa,addr_in_t *addr);
size_t      addr_un       (void    *sa,char const *path);
void        addr_un_out   (void    *sa,
                           size_t   sa_size,
                           char    *path,
                           size_t   path_size);

bool        addr_in_equal (addr_in_t a,addr_in_t b);
void        addr_in_fmt   (char *buf,size_t buf_size,addr_in_t addr);

char       *ipv4_fmt      (char *buf,size_t buf_size,uint32_t ip);
bool        ipv4_parse    (char const *buf,uint32_t *ip_out);

bool        ipv4_mask     (uint32_t ip,unsigned *mask_out);
uint32_t    ipv4_maskip   (unsigned mask);
bool        ipv4_insubnet (uint32_t ip,subnet_t p);
bool        ipv4_local    (uint32_t ip);

char       *subnet_fmt    (char *buf,size_t buf_size,subnet_t p);
bool        subnet_parse  (char const *buf,subnet_t *p_out);

#endif

