#include "hex.h"
#include <ctype.h>
#include <assert.h>

static bool nib_get(char c,uint8_t *n){
  c = toupper(c);

  if(c >= '0' && c <= '9'){
    *n = 0x0+(c-'0');
    return 1;
  }

  if(c >= 'A' && c <= 'F'){
    *n = 0xA+(c-'A');
    return 1;
  }

  return 0;
}

static char nib_set(uint8_t n){
  assert(n <= 0xF);
  return n <= 0x9 ? '0'+n : 'A'+(n-0xA);
}

bool hex_dec(uint8_t     *buf,
             size_t       buf_size,
             char const  *hex,
             size_t      *size_out){

  size_t size = 0;

  while(*hex){
    uint8_t u;
    uint8_t l;

    if(!buf_size)           return 0;
    if(!nib_get(*hex++,&u)) return 0;
    if(!nib_get(*hex++,&l)) return 0;

    *buf++ = u<<4 | l;
    buf_size--;
    size++;
  }

  if(size_out){
    *size_out = size;
  }
  return 1;
}
 
void hex_enc(char          *buf,
             size_t         buf_size,
             uint8_t const *data,
             size_t         data_size){

  (void)buf_size;
  assert(buf_size >= data_size*2+1); //FF+\0

  while(data_size){
    uint8_t byte;
    byte    = *data++;
    *buf++  = nib_set(byte >> 4);
    *buf++  = nib_set(byte &  0xF);
    data_size--;
  }

  *buf = 0;
}

